﻿IncludeFile "LZMA.pbi"

Procedure.i PackMemory_LZMA (*SourceBuff, *DestBuff, iSourceSize, *iDestSize, iCompressionLevel = 5, iDictSize = #LZMA_DICT_4M)
 ; *SourceBuff  = ptr to the source buffer
 ; *DestBuff    = ptr to the destination buffer
 ; iSourceSize  = size of the source buffer
 ; *iDestSize   = ptr to the size of the destination buffer (it will contain the packed size on exit)
 
 ; NOTE: the size of the destination buffer *should* be at least the size of the source + SizeOf(T_LZMA_HEADER)
 ; NOTE: iCompressionLevel must be between 0 (lowest) and 9 (highest) - default 5
 ; NOTE: iDictSize must be between 12 (1 << 12 = 4 KB) and 28 (1 << 28 = 256 MB) - default 4MB
 ; NOTE: the destination buffer will contain a T_LZMA_HEADER followed by the packed data
 ; NOTE: #LZMA_ERROR_OUTPUT_EOF means the file cannot be compressed (or the destination buffer is too small)
  
 Protected iResult, iDestSize, iOutPropsSize, *OutProps, qSourceLen.q   
 
 Protected Lc = 3 
 Protected Lp = 0
 Protected Pb = 2
 Protected Fb = 32
 Protected iNumThreads = 2
 
 iDestSize = PeekI(*iDestSize) - SizeOf(T_LZMA_HEADER)
 
 If iCompressionLevel < 0 Or iCompressionLevel > 9
    ProcedureReturn #LZMA_ERROR_PARAM
 EndIf
 
 If iDictSize < 12 Or iDictSize > 28 ; 4 KB to 256 MB
     ProcedureReturn #LZMA_ERROR_PARAM
 EndIf
 iDictSize = 1 << iDictSize ; default 22 = 4 MB
 
 ; Allocate memory for the properties section
 *OutProps = AllocateMemory(#LZMA_PROPS_SIZE) ; 5 bytes
 
 iOutPropsSize = #LZMA_PROPS_SIZE
 
 iResult = LzmaCompress (*DestBuff + SizeOf(T_LZMA_HEADER), @iDestSize, *SourceBuff, iSourceSize, *OutProps, @iOutPropsSize, iCompressionLevel, iDictSize, Lc,Lp,Pb,Fb, iNumThreads) 

 If iResult <> #LZMA_OK Or iDestSize = 0 Or iOutPropsSize = 0
    FreeMemory(*OutProps)
    ProcedureReturn iResult
 EndIf
 
 ; promote to quad
 qSourceLen = iSourceSize
 
 ; write the header for the destination buffer
 CopyMemory(*OutProps, *DestBuff, #LZMA_PROPS_SIZE)
 CopyMemory(@qSourceLen, *DestBuff + #LZMA_PROPS_SIZE, 8)
 
 ; return the size of the compressed buffer starting at *DestBuff
 PokeI(*iDestSize, iDestSize + SizeOf(T_LZMA_HEADER)) 
 
 FreeMemory(*OutProps)
    
 ProcedureReturn iResult
EndProcedure


Procedure.i UnpackMemory_LZMA (*SourceBuff, *DestBuff, iSourceSize, *iDestSize)
 ; *SourceBuff  = ptr to the source buffer
 ; *DestBuff    = ptr to the destination buffer
 ; iSourceSize  = size of the source buffer
 ; *iDestSize   = ptr to the size of the destination buffer (it will contain the unpacked size on exit)
  
 ; NOTE: the source buffer must contain a T_LZMA_HEADER followed by the packed data
 ; NOTE: memory required for decompression = dictionary size + 16 KB
 
 Protected iResult
 Protected iDestSize = PeekI(*iDestSize)
 Protected iOffSourceSize = iSourceSize - SizeOf(T_LZMA_HEADER )
 Protected *Props
   
 ; retrieve Props from header
 *Props = AllocateMemory(#LZMA_PROPS_SIZE) ; 5 bytes
 CopyMemory(*SourceBuff, *Props, #LZMA_PROPS_SIZE)
  
 iResult = LzmaUncompress (*DestBuff, @iDestSize, *SourceBuff + SizeOf(T_LZMA_HEADER), @iOffSourceSize, *Props, #LZMA_PROPS_SIZE)  
 
 PokeI(*iDestSize, iDestSize)
 
 FreeMemory(*Props)
 
 ProcedureReturn iResult
EndProcedure



; IDE Options = PureBasic 4.41 (Windows - x86)
; CursorPosition = 2
; FirstLine = 37
; Folding = -
; EnableXP
; EnableUser
; Executable = test.exe
; CPU = 1